"""
$Id: PSCImprovementProposalFolder.py,v 1.8 2005/04/02 00:55:48 optilude Exp $
"""

from AccessControl import ClassSecurityInfo

from Products.CMFCore import CMFCorePermissions

from Products.Archetypes.public import registerType
from Products.Archetypes.public import BaseBTreeFolder

from Products.PloneSoftwareCenter.config import PROJECTNAME
from Products.PloneSoftwareCenter.content.schemata import PSCImprovementProposalFolderSchema

from Products.CMFCore.utils import getToolByName

class PSCImprovementProposalFolder(BaseBTreeFolder):
    """A proposal container has proposals, and a view for the listing."""

    __implements__ = (BaseBTreeFolder.__implements__,)

    content_icon = 'improvementproposal_icon.gif'
    archetype_name = 'Roadmap Section'
    immediate_view = default_view = 'psc_roadmap'
    schema = PSCImprovementProposalFolderSchema

    global_allow = 0
    filter_content_types = 1
    allowed_content_types = ('PSCImprovementProposal',)

    security = ClassSecurityInfo()

    typeDescMsgId = 'description_edit_improvementproposalcontainer'
    typeDescription = ('A Roadmap Section is used to hold improvement '
                       'proposals. By default you have a single container '
                       'called "improvement", but you can other, more '
                       'specific ones.')

    actions = (
        {
            'id': 'view',
            'name': 'View',
            'action': 'string:${object_url}/psc_roadmap',
            'permissions': (CMFCorePermissions.View,),
        },
        {
            'id' : 'metadata',
            'name' : 'Properties',
            'action' : 'string:${object_url}/base_metadata',
            'permissions' : (CMFCorePermissions.ModifyPortalContent,),
            'visible' : 0,
        },
        {
            'id': 'sharing',
            'name': 'Sharing',
            'action': 'string:${object_url}/folder_localrole_form',
            'permissions': (CMFCorePermissions.ManageProperties,),
        },
    )

    security.declareProtected(CMFCorePermissions.AddPortalContent, 
                                'generateUniqueId')
    def generateUniqueId(self, type_name):
        """Generate a unique id, by finding the smallest unused number
        among the object in the container.

        This is used when creating objects with createObject.py, to ensure
        that items in this container gets sequentially numbered ids.
        """
        ids = self.objectIds()
        max = 1
        for id in ids:
            try:
                val = int(id)
                if val >= max:
                    max = val + 1
            except ValueError:
                continue
        return str(max)
        
    security.declareProtected(CMFCorePermissions.View, 'getStateTitle')
    def getStateTitle(self, state):
        """Given the state name, look up its title in portal_workflow
        """
        wftool = getToolByName(self, 'portal_workflow')
        wf = wftool.psc_improvementproposal_workflow
        return wf.states[state].title
        
    security.declareProtected(CMFCorePermissions.View, 
                                'getImprovementProposals')
    def getImprovementProposals(self, review_state = []):
        """Get the improvement proposals in this folder, sorted on plip number,
        as a list of catalog brains.
        """
        catalog = getToolByName(self, 'portal_catalog')
        
        query = {}
        query['portal_type'] = 'PSCImprovementProposal'
        query['path'] = '/'.join(self.getPhysicalPath())
        
        if review_state:
            query['review_state'] = review_state
        
        proposals = catalog.portal_catalog(query)
                                    
        proposals = [p for p in proposals]
        proposals.sort(lambda x, y: cmp(int(x.getId), int(y.getId)))
        return proposals

registerType(PSCImprovementProposalFolder, PROJECTNAME)
